/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributors:
 *  Chip Clark <chipc@netscape.com>
 *  Seth Spitzer <sspitzer@netscape.com>
 */

const kDefault = 0;
const kUserSet = 1;
const kLocked = 2;

const nsIAtomService = Components.interfaces.nsIAtomService;
const nsAtomService_CONTRACTID = "@mozilla.org/atom-service;1";

var prefEntries = new Array();
var atomService = Components.classes[nsAtomService_CONTRACTID].getService(nsIAtomService);
var gLockAtoms = [atomService.getAtom("default"), atomService.getAtom("user"), atomService.getAtom("locked")];
var gLockStrs = ["default", "user set","locked"]; 
var gPrefBranch;
var gTree;
var gPrefEditor = { editorPrefName : null, editorTextVal : null, editorBoolVal : null,
                    savePrefButton : null, defaultPrefButton : null};

const kStrType = 0;
const kIntType = 1;
const kBoolType = 2;
const nsIPrefBranch = Components.interfaces.nsIPrefBranch;

var gTypeStrs = {};
gTypeStrs[nsIPrefBranch.PREF_STRING] = "string";
gTypeStrs[nsIPrefBranch.PREF_BOOL]   =   "bool";
gTypeStrs[nsIPrefBranch.PREF_INT]    =    "int";

var view = ({
    rowCount : 0, 
    getCellText : function(k, col) {	
            if (!prefEntries[k])
                return "";
            
            var value;
            var prefEntry = prefEntries[k];
            var prefName = prefEntry.prefName;
            var strvalue;

            switch (col) {
              case "prefName":
                strvalue = prefName;
                break;
              case "lockCol":
                strvalue = getLockString(prefName);
                break;
              case "typeCol":
                strvalue = gTypeStrs[prefEntry.prefType];
                break;
              case "valueCol":
                strvalue = getPrefValue(prefName, prefEntry.prefType);
                break;
              case "typeCol":
                strvalue = gTypeStrs[value];
                break;
              default:
                strvalue = prefEntry[col];   
                break;
            }
            return strvalue;
        },
    getRowProperties : function(index, prop) {},
    getCellProperties : function(index, col, prop) {
      prop.AppendElement(gLockAtoms[getLockState(prefEntries[index].prefName)]);
    },
    getColumnProperties : function(col, elt, prop) {},
    treebox : null,
    selection : null,
    isContainer : function(index) { return false; },
    isContainerOpen : function(index) { return false; },
    isContainerEmpty : function(index) { return false; },
    isSorted : function() { },
    canDropOn : function(index) { return false; },
    canDropBeforeAfter : function(index,before) { return false; },
    drop : function(row,orientation) {},
    setTree : function(out) { this.treebox = out; },
    getParentIndex: function(rowIndex) { return -1 },
    hasNextSibling: function(rowIndex, afterIndex) { return false },
    getLevel: function(index) { return 1},
    getImageSrc: function(row, colID) { return ""},
    toggleOpenState : function(index) {},
    cycleHeader: function(colID, elt) {},
    selectionChanged : function() {
        updatePrefEditor(this.selection);
    },
    cycleCell: function(row, colID) {},
    isEditable: function(row, valueCol) {return true},
    isEditable: function(row, colID) {return false},
    setCellText: function(row, colID, value) {},
    performAction: function(action) {},
    performActionOnRow: function(action, row) {},
    performActionOnCell: function(action, row, colID) {},
    isSeparator: function(index) {return false}
});

function getLockState(prefName) {
    if (gPrefBranch.prefIsLocked(prefName))
        prefLockState = kLocked;
    else if(gPrefBranch.prefHasUserValue(prefName))
        prefLockState = kUserSet;
    else
        prefLockState = kDefault;
    return prefLockState;
}

function getLockString(prefName) {
    return gLockStrs[getLockState(prefName)];
}

function getPrefValue(prefName, prefType) {
    switch (prefType) {
    case nsIPrefBranch.PREF_INT:
        prefValue = "" + gPrefBranch.getIntPref(prefName);
        break;
    case nsIPrefBranch.PREF_BOOL:
        prefValue = gPrefBranch.getBoolPref(prefName) ? "true" : "false";
        break;
    case nsIPrefBranch.PREF_STRING:
        prefValue = htmlEscape(gPrefBranch.getCharPref(prefName));
        break;
    }
    return prefValue;
}

function onConfigLoad() {
    document.title = "Preference Editor";
    var prefCount = {value:0};
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    gPrefBranch = prefService.getBranch(null);
    var prefArray = gPrefBranch.getChildList("", prefCount);
    var prefName, prefType, prefIndex;
    var i = 0;
    var j = 0;   // This is used to avoid counting the "capability" preferences
    var k = 0;	 // This is to maintain a count of prefs (not including the "capability" prefs);

    prefArray.sort();
    for (i = 0; i < prefCount.value; i++) {
      if ((prefArray[i].indexOf("capability", 0) + 1) > 0) {  // avoid displaying "private" preferences
        j++;
        continue;
      }
      k = (i - j);   // avoid numbering "capability" prefs
      prefIndex = k + 1;
      prefArray[i] = htmlEscape(prefArray[i]);
      prefEntries[k] = { indexCol:prefIndex, prefName:prefArray[i], prefType:gPrefBranch.getPrefType(prefArray[i]) };
    }
    view.rowCount = k + 1;

    for (var id in gPrefEditor)
        gPrefEditor[id] = document.getElementById(id);

    gTree = document.getElementById("configTree");
    gTree.treeBoxObject.view = view;
    updatePrefEditor();
}

function htmlEscape(s) { 
    s = s.replace(/\&/g, "&amp;"); 
    s = s.replace(/\>/g, "&gt;"); 
    s = s.replace(/\</g, "&lt;"); 
    return s; 
}

function ConfigOnClick(event) {
    // we only care about button 0 (left click) events
    if (event.button != 0) return;

    var t = event.originalTarget;

    if (t.localName == "treecol") {
       HandleColumnClick(t.id);
    }
}

var gSortedColumn = "indexCol";
var gSortAscending = true;

function stringSortFunction(a,b) {
  var value;
  if (a<b)
    value = 1;
  else if (a>b) 
    value = -1;
  else
    value = 0;
 
  if (gSortAscending)
    return value;
  else
    return (value * -1);
}

function intSortFunction(a,b) {
  if (gSortAscending) 
    return (a - b);
  else 
    return (b - a);
}

function indexColSortFunction(x,y) {
  return intSortFunction(x.indexCol, y.indexCol);
}

function prefNameSortFunction(x,y) {
  return stringSortFunction(x.prefName, y.prefName);
}

function lockColSortFunction(x,y) {
  return stringSortFunction(getLockState(x.prefName), getLockState(y.prefName));
}

function typeColSortFunction(x,y) {
  return intSortFunction(x.prefType, y.prefType);
}

function valueColSortFunction(x,y) {
  return stringSortFunction(getPrefValue(x.prefName, x.prefType),
                            getPrefValue(y.prefName, y.prefType));
}

var gSortFunctions = {indexCol:indexColSortFunction, prefName:prefNameSortFunction, 
                      lockCol:lockColSortFunction, typeCol:typeColSortFunction, 
                      valueCol:valueColSortFunction};

function HandleColumnClick(id) {
  if (id == gSortedColumn) {
    gSortAscending = !gSortAscending;
    prefEntries.reverse();
  }
  else {
    prefEntries.sort(gSortFunctions[id]);
    gSortedColumn = id;
  }
  gTree.treeBoxObject.invalidate();
}

// when the user clicks the "Set Pref" button
function onSave() {
  var prefname = gPrefEditor.editorPrefName.getAttribute("value");
  var preftype = gPrefBranch.getPrefType(prefname);

  switch (preftype) {
  case nsIPrefBranch.PREF_STRING:
       gPrefBranch.setCharPref(prefname, gPrefEditor.editorTextVal.value);
       break;

  case nsIPrefBranch.PREF_INT:
       gPrefBranch.setIntPref(prefname, gPrefEditor.editorTextVal.value);
       break;

  case nsIPrefBranch.PREF_BOOL:
       var boolval = (gPrefEditor.editorBoolVal.selectedItem.value == "true");
       gPrefBranch.setBoolPref(prefname, boolval);
       break;
  }
}

function onDefault() {
  var prefname = gPrefEditor.editorPrefName.getAttribute("value");
  gPrefBranch.clearUserPref(prefname);
  updatePrefEditor(view.selection);
}

function updatePrefEditor(selection) {
  if (selection && selection.count == 1) {
    var rangemin = {};
    var rangemax = {};
    selection.getRangeAt(0, rangemin, rangemax);
    var prefindex = rangemin.value;
    var prefname = prefEntries[prefindex].prefName;
    var preftype = prefEntries[prefindex].prefType;
    updateEditControl(prefEntries[prefindex]);
  }
  else
    updateEditControl(null);
}

function updateEditControl(prefEntry) {
  if (!prefEntry) {
    gPrefEditor.editorPrefName.removeAttribute("value");
    gPrefEditor.savePrefButton.disabled = true;
    gPrefEditor.defaultPrefButton.disabled = true;
    gPrefEditor.editorTextVal.setAttribute("hidden","true");
    gPrefEditor.editorBoolVal.setAttribute("hidden","true");
    return;
  }
  var prefname = prefEntry.prefName;
  var preftype = gPrefBranch.getPrefType(prefname);
  gPrefEditor.editorPrefName.setAttribute("value", prefname);
  gPrefEditor.savePrefButton.disabled = false;
  gPrefEditor.defaultPrefButton.disabled = false;
    
  switch (preftype) {
    case nsIPrefBranch.PREF_STRING:
         gPrefEditor.editorTextVal.removeAttribute("hidden", "true");
         gPrefEditor.editorBoolVal.setAttribute("hidden","true");
         gPrefEditor.editorTextVal.value = gPrefBranch.getCharPref(prefname);
         break;

    case nsIPrefBranch.PREF_INT:
         gPrefEditor.editorTextVal.removeAttribute("hidden", "true");
         gPrefEditor.editorBoolVal.setAttribute("hidden","true");
         gPrefEditor.editorTextVal.value = gPrefBranch.getIntPref(prefname);
         break;

    case nsIPrefBranch.PREF_BOOL:
         gPrefEditor.editorBoolVal.removeAttribute("hidden", "true");
         gPrefEditor.editorTextVal.setAttribute("hidden","true");
         var index = gPrefBranch.getBoolPref(prefname) ? 0 : 1;
         gPrefEditor.editorBoolVal.selectedIndex = index;
         break;
        
    default: dump("Woops, unknown pref type " + preftype + "\n");
             break;
  }
}